/* boxLayout 1.0 Copyright (c) 2015 Douglas Henke mythopoeic.org/boxlayout */

/*** This is free software -- see COPYING for details.

For detailed information and examples, please see:

   http://mythopoeic.org/boxlayout

The web page above includes full (well, *some*) documentation and a
live version of the included example.

This is version 1.0, released on 04 June 2015.
***/

(function($) {
   /*** boxsize() -- get value of data-box-layout-size attribute, or 1

   Returns the numeric value of the data-box-layout-size attribute of an
   HTML element. If the element in question has no such attribute,
   returns 1.

   Arguments:
      elem -- HTML element as a jQuery object
   ***/
   function boxsize(elem) {
      var s;
      if(typeof((s= $(elem).data("boxLayoutSize"))) !== 'undefined') return s;
      return 1;
   }

   /*** outerSize() -- set outer dimensions of div (accounting for margins)

   Calling outerSize(elem, w, h) is the equivalent of:
      $(elem).outerWidth(w); $(elem).outerHeight(h);
   except that outerSize() also takes into account the margins of elem.

   Arguments:
      elem -- div to be modified (as a jQuery object)
      w -- overall width in pixels, including margins, border and padding
      h -- overall height in pixels, including all the above
   ***/
   function outerSize(elem, w, h) {
      var m1, m2;

      m1 = parseInt($(elem).css('margin-top'));
      m2 = parseInt($(elem).css('margin-bottom'));
      $(elem).outerHeight(h-m1-m2);
      m1 = parseInt($(elem).css('margin-left'));
      m2 = parseInt($(elem).css('margin-right'));
      $(elem).outerWidth(w-m1-m2);
   }

   /*** boxLayout() -- stack div elements horizontally or vertically

   Given a parent div element, finds all the immediate children which are
   also div elements, and stacks them so they exactly fill the parent div.

   For horizontal stacking, the top and bottom edges of all the children
   are made to match those of the parent. Each child is given a share of
   the parent width proportion to that child's boxsize() (which see).
   Children are stacked left to right, in the order they appear in the
   document.

   Vertical stacking is similar; left and right edges of children are
   made to match those of the parent. Child height is proportional to
   boxsize(), and stacking is top to bottom in document order.

   Example: Suppose there are two child div elements: A (boxsize() 3)
   and B (boxsize() 1). If I stack them horizontally, A will fill the
   left-hand 75% of the parent, and B the right-hand 25%.

   Arguments:
      id -- id of parent div, as a string
      vertical -- true for vertical stacking, false for horizontal; if
         omitted, defaults to false
   ***/
   $.boxLayout = function(id, vertical) {
      var i, sum, bsize, pos, elem;

      // use default for vertical argument if unset
      if(typeof(vertical) === 'undefined') vertical = false;

      // Let elem be the jQuery object whose HTML ID is id:
      elem = $('#'+id);

      /* Let size be the size of the parent element along the axis
         in which we're stacking. Let osize be the size of the parent
         element along the other axis. */
      if(vertical) {
         size = $(elem).outerHeight(true);
         osize = $(elem).outerWidth(true);
      } else {
         size = $(elem).outerWidth(true);
         osize = $(elem).outerHeight(true);
      }

      /* Let sum be the sum of the boxsize() across all the child div
         elements we're stacking: */
      sum = 0;
      $(elem).find('>div').each(function() { sum += boxsize(this); });

      /* pos tracks the position of the next child div element in the stack
         (relative to the parent). For horizontal stacks, pos is the position
         of the left edge. For vertical stacks, it is the position of the
         top edge. */
      pos = 0;
      $(elem).find('>div').each(function() {
         $(this).css('position', 'absolute');

         /* Let bsize be the size of the child div element along the axis
            in which we're stacking. That is: a fraction of the size of
            the parent div along the same axis, proportional to the
            boxsize() of the child. */
         bsize = boxsize(this) * size / sum;

         // Set the size and position of the child element:
         if(vertical) {
            $(this).css({ top: pos, left: 'auto' });
            outerSize(this, osize, bsize);
         } else {
            $(this).css({ top: 'auto', left: pos });
            outerSize(this, bsize, osize);
         }

         /* For horizontal stacking, the left edge of the next child div
            lines up with the right edge of the child we just positioned.
            For vertical stacking, the current bottom is the next top. */
         pos += bsize;
      });
   };
}(jQuery));
